"""The 'paint' modules provides factory functions for modular painting
operations. """
#  MIT Licence
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._abstract_paint import AbstractPaint
from ._border_rect import BorderRect
from ._fill_rect import FillRect
from ._data_paint import DataPaint
from ._text_rect import TextRect
